@@ -1,7 +1,8 @@
-//  $Id: physic.cpp 598 2004-04-20 18:39:18Z grumbel $
-// 
+//  $Id: physic.cpp 4082 2006-07-24 19:27:22Z sommer $
+//
 //  SuperTux
 //  Copyright (C) 2004 Tobias Glaesser <tobi.web@gmx.de>
+//  Copyright (C) 2006 Matthias Braun <matze@braunis.de>
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -12,23 +13,17 @@
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
-// 
+//
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //  02111-1307, USA.
+#include <config.h>
 
-#include <stdio.h>
-
-#include "scene.h"
-#include "defines.h"
-#include "physic.h"
-#include "timer.h"
-#include "world.h"
-#include "level.h"
+#include "physic.hpp"
 
 Physic::Physic()
-    : ax(0), ay(0), vx(0), vy(0), gravity_enabled(true)
+    : ax(0), ay(0), vx(0), vy(0), gravity_enabled_flag(true), gravity(1000)
 {
 }
 
@@ -40,7 +35,7 @@ void
 Physic::reset()
 {
     ax = ay = vx = vy = 0;
-    gravity_enabled = true;
+    gravity_enabled_flag = true;
 }
 
 void
@@ -52,36 +47,49 @@ Physic::set_velocity_x(float nvx)
 void
 Physic::set_velocity_y(float nvy)
 {
-  vy = -nvy;
+  vy = nvy;
 }
 
 void
 Physic::set_velocity(float nvx, float nvy)
 {
   vx = nvx;
-  vy = -nvy;
+  vy = nvy;
+}
+
+void
+Physic::set_velocity(const Vector& vector)
+{
+  vx = vector.x;
+  vy = vector.y;
 }
 
 void Physic::inverse_velocity_x()
 {
-vx = -vx;
+  vx = -vx;
 }
 
 void Physic::inverse_velocity_y()
 {
-vy = -vy;
+  vy = -vy;
 }
 
 float
-Physic::get_velocity_x()
+Physic::get_velocity_x() const
 {
     return vx;
 }
 
 float
-Physic::get_velocity_y()
+Physic::get_velocity_y() const
 {
-    return -vy;
+    return vy;
+}
+
+Vector
+Physic::get_velocity() const
+{
+  return Vector(vx, vy);
 }
 
 void
@@ -93,46 +101,69 @@ Physic::set_acceleration_x(float nax)
 void
 Physic::set_acceleration_y(float nay)
 {
-  ay = -nay;
+  ay = nay;
 }
 
 void
 Physic::set_acceleration(float nax, float nay)
 {
-    ax = nax;
-    ay = -nay;
+  ax = nax;
+  ay = nay;
 }
 
 float
-Physic::get_acceleration_x()
+Physic::get_acceleration_x() const
 {
-    return ax;
+  return ax;
 }
 
 float
-Physic::get_acceleration_y()
+Physic::get_acceleration_y() const
 {
-    return -ay;
+  return ay;
+}
+
+Vector
+Physic::get_acceleration() const
+{
+  return Vector(ax, ay);
 }
 
 void
 Physic::enable_gravity(bool enable_gravity)
 {
-  gravity_enabled = enable_gravity;
+  gravity_enabled_flag = enable_gravity;
+}
+
+bool
+Physic::gravity_enabled() const
+{
+  return gravity_enabled_flag;
 }
 
 void
-Physic::apply(float frame_ratio, float &x, float &y)
+Physic::set_gravity(float gravity)
 {
-  float gravity = World::current()->get_level()->gravity;
-  float grav;
-  if(gravity_enabled)
-    grav = gravity / 100.0;
-  else
-    grav = 0;
-
-  x += vx * frame_ratio + ax * frame_ratio * frame_ratio;
-  y += vy * frame_ratio + (ay + grav) * frame_ratio * frame_ratio;
-  vx += ax * frame_ratio;
-  vy += (ay + grav) * frame_ratio;
+  this->gravity = gravity;
+}
+
+float
+Physic::get_gravity() const
+{
+  return gravity;
+}
+
+Vector
+Physic::get_movement(float elapsed_time)
+{
+  float grav = gravity_enabled_flag ? gravity : 0;
+
+  Vector result(
+      vx * elapsed_time + ax * elapsed_time * elapsed_time,
+      vy * elapsed_time + (ay + grav) * elapsed_time * elapsed_time
+  );
+  vx += ax * elapsed_time;
+  vy += (ay + grav) * elapsed_time;
+
+  return result;
 }
